@@ -26,5 +26,23 @@ cpu_count <- function() {
 #' @param num_threads integer: New number of threads for thread pool
 #' @export
 set_cpu_count <- function(num_threads) {
+  current_cpu_count <- cpu_count()
   SetCpuThreadPoolCapacity(as.integer(num_threads))
+  invisible(current_cpu_count)
+}
+
+#' Manage the global I/O thread pool in libarrow
+#'
+#' @export
+io_thread_count <- function() {
+  GetIOThreadPoolCapacity()
+}
+
+#' @rdname io_thread_count
+#' @param num_threads integer: New number of threads for thread pool
+#' @export
+set_io_thread_count <- function(num_threads) {
+  current_io_thread_count <- io_thread_count()
+  SetIOThreadPoolCapacity(as.integer(num_threads))
+  invisible(current_io_thread_count)
 }
