@@ -55,6 +55,7 @@ ${CMAKE} -DARROW_BOOST_USE_SHARED=OFF \
     -DARROW_BUILD_TESTS=OFF \
     -DARROW_BUILD_SHARED=OFF \
     -DARROW_BUILD_STATIC=ON \
+    -DARROW_ACERO=${ARROW_ACERO:-$ARROW_DEFAULT_PARAM} \
     -DARROW_COMPUTE=ON \
     -DARROW_CSV=ON \
     -DARROW_DATASET=${ARROW_DATASET:-ON} \
@@ -86,7 +87,7 @@ ${CMAKE} -DARROW_BOOST_USE_SHARED=OFF \
     -Dxsimd_SOURCE=${xsimd_SOURCE:-} \
     -Dzstd_SOURCE=${zstd_SOURCE:-} \
     ${EXTRA_CMAKE_FLAGS} \
-    -G ${CMAKE_GENERATOR:-"Unix Makefiles"} \
+    -G "${CMAKE_GENERATOR:-Unix Makefiles}" \
     ${SOURCE_DIR}
 
 ${CMAKE} --build . --target install -- -j $N_JOBS
