@@ -34,12 +34,16 @@
 #include <arrow/compute/type_fwd.h>
 #include <arrow/csv/type_fwd.h>
 
+#if defined(ARROW_R_WITH_ACERO)
+#include <arrow/acero/options.h>
+#include <arrow/acero/type_fwd.h>
+namespace acero = ::arrow::acero;
+#endif
+
 #if defined(ARROW_R_WITH_DATASET)
 #include <arrow/dataset/type_fwd.h>
 #endif
 
-#include <arrow/compute/exec/options.h>
-
 #include <arrow/filesystem/type_fwd.h>
 #include <arrow/io/type_fwd.h>
 #include <arrow/ipc/type_fwd.h>
@@ -51,15 +55,6 @@
 #include <arrow/type_fwd.h>
 #include <arrow/util/type_fwd.h>
 
-namespace arrow {
-namespace compute {
-
-class ExecPlan;
-class ExecNode;
-
-}  // namespace compute
-}  // namespace arrow
-
 class ExecPlanReader;
 
 #if defined(ARROW_R_WITH_PARQUET)
@@ -109,9 +104,14 @@ static inline void StopIfNotOk(const Status& status) {
     if (unwind_detail) {
       throw cpp11::unwind_exception(unwind_detail->token);
     } else {
-      // ARROW-13039: be careful not to interpret our error message as a %-format string
+      // We need to translate this to "native" encoding for the error to be
+      // displayed properly using cpp11::stop()
       std::string s = status.ToString();
-      cpp11::stop("%s", s.c_str());
+      cpp11::strings s_utf8 = cpp11::as_sexp(s);
+      const char* s_native = cpp11::safe[Rf_translateChar](s_utf8[0]);
+
+      // ARROW-13039: be careful not to interpret our error message as a %-format string
+      cpp11::stop("%s", s_native);
     }
   }
 }
@@ -234,6 +234,7 @@ void Init_Altrep_classes(DllInfo* dll);
 
 SEXP MakeAltrepVector(const std::shared_ptr<ChunkedArray>& chunked_array);
 bool is_arrow_altrep(SEXP x);
+bool is_unmaterialized_arrow_altrep(SEXP x);
 std::shared_ptr<ChunkedArray> vec_to_arrow_altrep_bypass(SEXP);
 
 }  // namespace altrep
