@@ -20,6 +20,7 @@
 
 #include <arrow/filesystem/filesystem.h>
 #include <arrow/filesystem/localfs.h>
+#include <arrow/util/key_value_metadata.h>
 
 namespace fs = ::arrow::fs;
 namespace io = ::arrow::io;
@@ -350,6 +351,13 @@ std::string fs___S3FileSystem__region(const std::shared_ptr<fs::S3FileSystem>& f
 
 #endif
 
+// [[arrow::export]]
+void FinalizeS3() {
+#if defined(ARROW_R_WITH_S3)
+  StopIfNotOk(fs::FinalizeS3());
+#endif
+}
+
 #if defined(ARROW_R_WITH_GCS)
 
 #include <arrow/filesystem/gcsfs.h>
@@ -422,4 +430,64 @@ std::shared_ptr<fs::GcsFileSystem> fs___GcsFileSystem__Make(bool anonymous,
   return fs::GcsFileSystem::Make(gcs_opts, io_context);
 }
 
+// [[gcs::export]]
+cpp11::list fs___GcsFileSystem__options(const std::shared_ptr<fs::GcsFileSystem>& fs) {
+  using cpp11::literals::operator"" _nm;
+
+  cpp11::writable::list out;
+
+  fs::GcsOptions opts = fs->options();
+
+  // GcsCredentials
+  out.push_back({"anonymous"_nm = opts.credentials.anonymous()});
+
+  if (opts.credentials.access_token() != "") {
+    out.push_back({"access_token"_nm = opts.credentials.access_token()});
+  }
+
+  if (opts.credentials.expiration().time_since_epoch().count() != 0) {
+    out.push_back({"expiration"_nm = cpp11::as_sexp<double>(
+                       opts.credentials.expiration().time_since_epoch().count())});
+  }
+
+  if (opts.credentials.target_service_account() != "") {
+    out.push_back(
+        {"target_service_account"_nm = opts.credentials.target_service_account()});
+  }
+
+  if (opts.credentials.json_credentials() != "") {
+    out.push_back({"json_credentials"_nm = opts.credentials.json_credentials()});
+  }
+
+  // GcsOptions direct members
+  if (opts.endpoint_override != "") {
+    out.push_back({"endpoint_override"_nm = opts.endpoint_override});
+  }
+
+  if (opts.scheme != "") {
+    out.push_back({"scheme"_nm = opts.scheme});
+  }
+
+  if (opts.default_bucket_location != "") {
+    out.push_back({"default_bucket_location"_nm = opts.default_bucket_location});
+  }
+
+  out.push_back({"retry_limit_seconds"_nm = opts.retry_limit_seconds.value()});
+
+  // default_metadata
+  if (opts.default_metadata != nullptr && opts.default_metadata->size() > 0) {
+    cpp11::writable::strings metadata(opts.default_metadata->size());
+
+    metadata.names() = opts.default_metadata->keys();
+
+    for (int64_t i = 0; i < opts.default_metadata->size(); i++) {
+      metadata[static_cast<size_t>(i)] = opts.default_metadata->value(i);
+    }
+
+    out.push_back({"default_metadata"_nm = metadata});
+  }
+
+  return out;
+}
+
 #endif
