@@ -17,8 +17,6 @@
 
 #include "./arrow_types.h"
 
-#if defined(ARROW_R_WITH_ARROW)
-
 #include <arrow/array/array_base.h>
 #include <arrow/io/file.h>
 #include <arrow/io/memory.h>
@@ -29,13 +27,13 @@
 #include <arrow/util/key_value_metadata.h>
 
 // [[arrow::export]]
-int RecordBatch__num_columns(const std::shared_ptr<arrow::RecordBatch>& x) {
-  return x->num_columns();
+r_vec_size RecordBatch__num_columns(const std::shared_ptr<arrow::RecordBatch>& x) {
+  return r_vec_size(x->num_columns());
 }
 
 // [[arrow::export]]
-int RecordBatch__num_rows(const std::shared_ptr<arrow::RecordBatch>& x) {
-  return x->num_rows();
+r_vec_size RecordBatch__num_rows(const std::shared_ptr<arrow::RecordBatch>& x) {
+  return r_vec_size(x->num_rows());
 }
 
 // [[arrow::export]]
@@ -309,9 +307,7 @@ std::shared_ptr<arrow::RecordBatch> RecordBatch__from_arrays(SEXP schema_sxp, SE
 }
 
 // [[arrow::export]]
-int64_t RecordBatch__ReferencedBufferSize(
+r_vec_size RecordBatch__ReferencedBufferSize(
     const std::shared_ptr<arrow::RecordBatch>& batch) {
-  return ValueOrStop(arrow::util::ReferencedBufferSize(*batch));
+  return r_vec_size(ValueOrStop(arrow::util::ReferencedBufferSize(*batch)));
 }
-
-#endif
