@@ -54,7 +54,7 @@ struct DictionaryValue {
 
 template <typename T>
 struct DictionaryValue<T, enable_if_base_binary<T>> {
-  using type = util::string_view;
+  using type = std::string_view;
   using PhysicalType =
       typename std::conditional<std::is_same<typename T::offset_type, int32_t>::value,
                                 BinaryType, LargeBinaryType>::type;
@@ -62,7 +62,7 @@ struct DictionaryValue<T, enable_if_base_binary<T>> {
 
 template <typename T>
 struct DictionaryValue<T, enable_if_fixed_size_binary<T>> {
-  using type = util::string_view;
+  using type = std::string_view;
   using PhysicalType = BinaryType;
 };
 
@@ -112,8 +112,8 @@ class ARROW_EXPORT DictionaryMemoTable {
   Status GetOrInsert(const FloatType*, float value, int32_t* out);
   Status GetOrInsert(const DoubleType*, double value, int32_t* out);
 
-  Status GetOrInsert(const BinaryType*, util::string_view value, int32_t* out);
-  Status GetOrInsert(const LargeBinaryType*, util::string_view value, int32_t* out);
+  Status GetOrInsert(const BinaryType*, std::string_view value, int32_t* out);
+  Status GetOrInsert(const LargeBinaryType*, std::string_view value, int32_t* out);
 
   class DictionaryMemoTableImpl;
   std::unique_ptr<DictionaryMemoTableImpl> impl_;
@@ -146,24 +146,26 @@ class DictionaryBuilderBase : public ArrayBuilder {
                                         !is_fixed_size_binary_type<T1>::value,
                                     const std::shared_ptr<DataType>&>
                             value_type,
-                        MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+                        MemoryPool* pool = default_memory_pool(),
+                        int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(-1),
-        indices_builder_(start_int_size, pool),
+        indices_builder_(start_int_size, pool, alignment),
         value_type_(value_type) {}
 
   template <typename T1 = T>
   explicit DictionaryBuilderBase(
       enable_if_t<!is_fixed_size_binary_type<T1>::value, const std::shared_ptr<DataType>&>
           value_type,
-      MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+      MemoryPool* pool = default_memory_pool(),
+      int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(-1),
-        indices_builder_(pool),
+        indices_builder_(pool, alignment),
         value_type_(value_type) {}
 
   template <typename T1 = T>
@@ -171,12 +173,13 @@ class DictionaryBuilderBase : public ArrayBuilder {
       const std::shared_ptr<DataType>& index_type,
       enable_if_t<!is_fixed_size_binary_type<T1>::value, const std::shared_ptr<DataType>&>
           value_type,
-      MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+      MemoryPool* pool = default_memory_pool(),
+      int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(-1),
-        indices_builder_(index_type, pool),
+        indices_builder_(index_type, pool, alignment),
         value_type_(value_type) {}
 
   template <typename B = BuilderType, typename T1 = T>
@@ -185,35 +188,38 @@ class DictionaryBuilderBase : public ArrayBuilder {
                                         is_fixed_size_binary_type<T1>::value,
                                     const std::shared_ptr<DataType>&>
                             value_type,
-                        MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+                        MemoryPool* pool = default_memory_pool(),
+                        int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(static_cast<const T1&>(*value_type).byte_width()),
-        indices_builder_(start_int_size, pool),
+        indices_builder_(start_int_size, pool, alignment),
         value_type_(value_type) {}
 
   template <typename T1 = T>
   explicit DictionaryBuilderBase(
       enable_if_fixed_size_binary<T1, const std::shared_ptr<DataType>&> value_type,
-      MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+      MemoryPool* pool = default_memory_pool(),
+      int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(static_cast<const T1&>(*value_type).byte_width()),
-        indices_builder_(pool),
+        indices_builder_(pool, alignment),
         value_type_(value_type) {}
 
   template <typename T1 = T>
   explicit DictionaryBuilderBase(
       const std::shared_ptr<DataType>& index_type,
       enable_if_fixed_size_binary<T1, const std::shared_ptr<DataType>&> value_type,
-      MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+      MemoryPool* pool = default_memory_pool(),
+      int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, value_type)),
         delta_offset_(0),
         byte_width_(static_cast<const T1&>(*value_type).byte_width()),
-        indices_builder_(index_type, pool),
+        indices_builder_(index_type, pool, alignment),
         value_type_(value_type) {}
 
   template <typename T1 = T>
@@ -223,12 +229,13 @@ class DictionaryBuilderBase : public ArrayBuilder {
 
   // This constructor doesn't check for errors. Use InsertMemoValues instead.
   explicit DictionaryBuilderBase(const std::shared_ptr<Array>& dictionary,
-                                 MemoryPool* pool = default_memory_pool())
-      : ArrayBuilder(pool),
+                                 MemoryPool* pool = default_memory_pool(),
+                                 int64_t alignment = kDefaultBufferAlignment)
+      : ArrayBuilder(pool, alignment),
         memo_table_(new internal::DictionaryMemoTable(pool, dictionary)),
         delta_offset_(0),
         byte_width_(-1),
-        indices_builder_(pool),
+        indices_builder_(pool, alignment),
         value_type_(dictionary->type()) {}
 
   ~DictionaryBuilderBase() override = default;
@@ -257,13 +264,13 @@ class DictionaryBuilderBase : public ArrayBuilder {
   /// \brief Append a fixed-width string (only for FixedSizeBinaryType)
   template <typename T1 = T>
   enable_if_fixed_size_binary<T1, Status> Append(const uint8_t* value) {
-    return Append(util::string_view(reinterpret_cast<const char*>(value), byte_width_));
+    return Append(std::string_view(reinterpret_cast<const char*>(value), byte_width_));
   }
 
   /// \brief Append a fixed-width string (only for FixedSizeBinaryType)
   template <typename T1 = T>
   enable_if_fixed_size_binary<T1, Status> Append(const char* value) {
-    return Append(util::string_view(value, byte_width_));
+    return Append(std::string_view(value, byte_width_));
   }
 
   /// \brief Append a string (only for binary types)
@@ -275,13 +282,13 @@ class DictionaryBuilderBase : public ArrayBuilder {
   /// \brief Append a string (only for binary types)
   template <typename T1 = T>
   enable_if_binary_like<T1, Status> Append(const char* value, int32_t length) {
-    return Append(util::string_view(value, length));
+    return Append(std::string_view(value, length));
   }
 
   /// \brief Append a string (only for string types)
   template <typename T1 = T>
   enable_if_string_like<T1, Status> Append(const char* value, int32_t length) {
-    return Append(util::string_view(value, length));
+    return Append(std::string_view(value, length));
   }
 
   /// \brief Append a decimal (only for Decimal128Type)
@@ -366,10 +373,11 @@ class DictionaryBuilderBase : public ArrayBuilder {
     return Status::OK();
   }
 
-  Status AppendArraySlice(const ArrayData& array, int64_t offset, int64_t length) final {
+  Status AppendArraySlice(const ArraySpan& array, int64_t offset, int64_t length) final {
     // Visit the indices and insert the unpacked values.
     const auto& dict_ty = internal::checked_cast<const DictionaryType&>(*array.type);
-    const typename TypeTraits<T>::ArrayType dict(array.dictionary);
+    // See if possible to avoid using ToArrayData here
+    const typename TypeTraits<T>::ArrayType dict(array.dictionary().ToArrayData());
     ARROW_RETURN_NOT_OK(Reserve(length));
     switch (dict_ty.index_type()->id()) {
       case Type::UINT8:
@@ -490,10 +498,10 @@ class DictionaryBuilderBase : public ArrayBuilder {
  protected:
   template <typename c_type>
   Status AppendArraySliceImpl(const typename TypeTraits<T>::ArrayType& dict,
-                              const ArrayData& array, int64_t offset, int64_t length) {
+                              const ArraySpan& array, int64_t offset, int64_t length) {
     const c_type* values = array.GetValues<c_type>(1) + offset;
     return VisitBitBlocks(
-        array.buffers[0], array.offset + offset, length,
+        array.buffers[0].data, array.offset + offset, length,
         [&](const int64_t position) {
           const int64_t index = static_cast<int64_t>(values[position]);
           if (dict.IsValid(index)) {
