@@ -22,7 +22,8 @@
 #include <algorithm>
 #include <cstdint>
 
-#include "arrow/compute/exec/util.h"
+#include "arrow/compute/util.h"
+#include "arrow/compute/util_internal.h"
 #include "arrow/util/bit_util.h"
 #include "arrow/util/ubsan.h"
 
