@@ -17,7 +17,7 @@
 
 #include "arrow/dataset/file_base.h"
 
-#include <arrow/compute/exec/exec_plan.h>
+#include "arrow/acero/exec_plan.h"
 
 #include <algorithm>
 #include <atomic>
@@ -26,14 +26,15 @@
 #include <variant>
 #include <vector>
 
+#include "arrow/acero/map_node.h"
+#include "arrow/acero/query_context.h"
+#include "arrow/acero/util.h"
 #include "arrow/compute/api_scalar.h"
-#include "arrow/compute/exec/forest_internal.h"
-#include "arrow/compute/exec/map_node.h"
-#include "arrow/compute/exec/query_context.h"
-#include "arrow/compute/exec/subtree_internal.h"
 #include "arrow/dataset/dataset_internal.h"
 #include "arrow/dataset/dataset_writer.h"
+#include "arrow/dataset/forest_internal.h"
 #include "arrow/dataset/scanner.h"
+#include "arrow/dataset/subtree_internal.h"
 #include "arrow/filesystem/filesystem.h"
 #include "arrow/filesystem/path_util.h"
 #include "arrow/io/compressed.h"
@@ -210,7 +211,7 @@ bool FileFragment::Equals(const FileFragment& other) const {
 
 struct FileSystemDataset::FragmentSubtrees {
   // Forest for skipping fragments based on extracted subtree expressions
-  compute::Forest forest;
+  Forest forest;
   // fragment indices and subtree expressions in forest order
   std::vector<std::variant<int, compute::Expression>> fragments_and_subtrees;
 };
@@ -267,13 +268,13 @@ std::string FileSystemDataset::ToString() const {
 
 void FileSystemDataset::SetupSubtreePruning() {
   subtrees_ = std::make_shared<FragmentSubtrees>();
-  compute::SubtreeImpl impl;
+  SubtreeImpl impl;
 
   auto encoded = impl.EncodeGuarantees(
       [&](int index) { return fragments_[index]->partition_expression(); },
       static_cast<int>(fragments_.size()));
 
-  std::sort(encoded.begin(), encoded.end(), compute::SubtreeImpl::ByGuarantee());
+  std::sort(encoded.begin(), encoded.end(), SubtreeImpl::ByGuarantee());
 
   for (const auto& e : encoded) {
     if (e.index) {
@@ -283,8 +284,8 @@ void FileSystemDataset::SetupSubtreePruning() {
     }
   }
 
-  subtrees_->forest = compute::Forest(static_cast<int>(encoded.size()),
-                                      compute::SubtreeImpl::IsAncestor{encoded});
+  subtrees_->forest =
+      Forest(static_cast<int>(encoded.size()), SubtreeImpl::IsAncestor{encoded});
 }
 
 Result<FragmentIterator> FileSystemDataset::GetFragmentsImpl(
@@ -298,7 +299,7 @@ Result<FragmentIterator> FileSystemDataset::GetFragmentsImpl(
 
   std::vector<compute::Expression> predicates{predicate};
   RETURN_NOT_OK(subtrees_->forest.Visit(
-      [&](compute::Forest::Ref ref) -> Result<bool> {
+      [&](Forest::Ref ref) -> Result<bool> {
         if (auto fragment_index =
                 std::get_if<int>(&subtrees_->fragments_and_subtrees[ref.i])) {
           fragment_indices.push_back(*fragment_index);
@@ -317,7 +318,7 @@ Result<FragmentIterator> FileSystemDataset::GetFragmentsImpl(
         predicates.push_back(std::move(simplified));
         return true;
       },
-      [&](compute::Forest::Ref ref) { predicates.pop_back(); }));
+      [&](Forest::Ref ref) { predicates.pop_back(); }));
 
   std::sort(fragment_indices.begin(), fragment_indices.end());
 
@@ -384,7 +385,7 @@ Status WriteBatch(
   return Status::OK();
 }
 
-class DatasetWritingSinkNodeConsumer : public compute::SinkNodeConsumer {
+class DatasetWritingSinkNodeConsumer : public acero::SinkNodeConsumer {
  public:
   DatasetWritingSinkNodeConsumer(std::shared_ptr<const KeyValueMetadata> custom_metadata,
                                  FileSystemDatasetWriteOptions write_options)
@@ -392,8 +393,8 @@ class DatasetWritingSinkNodeConsumer : public compute::SinkNodeConsumer {
         write_options_(std::move(write_options)) {}
 
   Status Init(const std::shared_ptr<Schema>& schema,
-              compute::BackpressureControl* backpressure_control,
-              compute::ExecPlan* plan) override {
+              acero::BackpressureControl* backpressure_control,
+              acero::ExecPlan* plan) override {
     if (custom_metadata_) {
       schema_ = schema->WithMetadata(custom_metadata_);
     } else {
@@ -454,19 +455,19 @@ Status FileSystemDataset::Write(const FileSystemDatasetWriteOptions& write_optio
   // when reading from a single input file.
   const auto& custom_metadata = scanner->options()->projected_schema->metadata();
 
-  compute::Declaration plan = compute::Declaration::Sequence({
+  acero::Declaration plan = acero::Declaration::Sequence({
       {"scan", ScanNodeOptions{dataset, scanner->options()}},
-      {"filter", compute::FilterNodeOptions{scanner->options()->filter}},
-      {"project", compute::ProjectNodeOptions{std::move(exprs), std::move(names)}},
+      {"filter", acero::FilterNodeOptions{scanner->options()->filter}},
+      {"project", acero::ProjectNodeOptions{std::move(exprs), std::move(names)}},
       {"write", WriteNodeOptions{write_options, custom_metadata}},
   });
 
-  return compute::DeclarationToStatus(std::move(plan), scanner->options()->use_threads);
+  return acero::DeclarationToStatus(std::move(plan), scanner->options()->use_threads);
 }
 
-Result<compute::ExecNode*> MakeWriteNode(compute::ExecPlan* plan,
-                                         std::vector<compute::ExecNode*> inputs,
-                                         const compute::ExecNodeOptions& options) {
+Result<acero::ExecNode*> MakeWriteNode(acero::ExecPlan* plan,
+                                       std::vector<acero::ExecNode*> inputs,
+                                       const acero::ExecNodeOptions& options) {
   if (inputs.size() != 1) {
     return Status::Invalid("Write SinkNode requires exactly 1 input, got ",
                            inputs.size());
@@ -487,17 +488,17 @@ Result<compute::ExecNode*> MakeWriteNode(compute::ExecPlan* plan,
 
   ARROW_ASSIGN_OR_RAISE(
       auto node,
-      compute::MakeExecNode("consuming_sink", plan, std::move(inputs),
-                            compute::ConsumingSinkNodeOptions{std::move(consumer)}));
+      acero::MakeExecNode("consuming_sink", plan, std::move(inputs),
+                          acero::ConsumingSinkNodeOptions{std::move(consumer)}));
 
   return node;
 }
 
 namespace {
 
-class TeeNode : public compute::MapNode {
+class TeeNode : public acero::MapNode {
  public:
-  TeeNode(compute::ExecPlan* plan, std::vector<compute::ExecNode*> inputs,
+  TeeNode(acero::ExecPlan* plan, std::vector<acero::ExecNode*> inputs,
           std::shared_ptr<Schema> output_schema,
           FileSystemDatasetWriteOptions write_options)
       : MapNode(plan, std::move(inputs), std::move(output_schema)),
@@ -512,9 +513,9 @@ class TeeNode : public compute::MapNode {
     return MapNode::StartProducing();
   }
 
-  static Result<compute::ExecNode*> Make(compute::ExecPlan* plan,
-                                         std::vector<compute::ExecNode*> inputs,
-                                         const compute::ExecNodeOptions& options) {
+  static Result<acero::ExecNode*> Make(acero::ExecPlan* plan,
+                                       std::vector<acero::ExecNode*> inputs,
+                                       const acero::ExecNodeOptions& options) {
     RETURN_NOT_OK(ValidateExecNodeInputs(plan, inputs, 1, "TeeNode"));
 
     const WriteNodeOptions write_node_options =
@@ -528,15 +529,9 @@ class TeeNode : public compute::MapNode {
 
   const char* kind_name() const override { return "TeeNode"; }
 
-  void Finish(Status finish_st) override {
-    if (!finish_st.ok()) {
-      MapNode::Finish(std::move(finish_st));
-      return;
-    }
-    dataset_writer_->Finish();
-  }
+  void Finish() override { dataset_writer_->Finish(); }
 
-  Result<compute::ExecBatch> DoTee(const compute::ExecBatch& batch) {
+  Result<compute::ExecBatch> ProcessBatch(compute::ExecBatch batch) override {
     ARROW_ASSIGN_OR_RAISE(std::shared_ptr<RecordBatch> record_batch,
                           batch.ToRecordBatch(output_schema()));
     ARROW_RETURN_NOT_OK(WriteNextBatch(std::move(record_batch), batch.guarantee));
@@ -548,30 +543,12 @@ class TeeNode : public compute::MapNode {
     return WriteBatch(batch, guarantee, write_options_,
                       [this](std::shared_ptr<RecordBatch> next_batch,
                              const PartitionPathFormat& destination) {
-                        util::tracing::Span span;
                         dataset_writer_->WriteRecordBatch(
                             next_batch, destination.directory, destination.filename);
                         return Status::OK();
                       });
   }
 
-  void InputReceived(compute::ExecNode* input, compute::ExecBatch batch) override {
-    EVENT(span_, "InputReceived", {{"batch.length", batch.length}});
-    DCHECK_EQ(input, inputs_[0]);
-    auto func = [this](compute::ExecBatch batch) {
-      util::tracing::Span span;
-      START_SPAN_WITH_PARENT(span, span_, "InputReceived",
-                             {{"tee", ToStringExtra()},
-                              {"node.label", label()},
-                              {"batch.length", batch.length}});
-      auto result = DoTee(std::move(batch));
-      MARK_SPAN(span, result.status());
-      END_SPAN(span);
-      return result;
-    };
-    this->SubmitTask(std::move(func), std::move(batch));
-  }
-
   void Pause() { inputs_[0]->PauseProducing(this, ++backpressure_counter_); }
 
   void Resume() { inputs_[0]->ResumeProducing(this, ++backpressure_counter_); }
@@ -590,7 +567,7 @@ class TeeNode : public compute::MapNode {
 }  // namespace
 
 namespace internal {
-void InitializeDatasetWriter(arrow::compute::ExecFactoryRegistry* registry) {
+void InitializeDatasetWriter(arrow::acero::ExecFactoryRegistry* registry) {
   DCHECK_OK(registry->AddFactory("write", MakeWriteNode));
   DCHECK_OK(registry->AddFactory("tee", TeeNode::Make));
 }
