@@ -29,6 +29,7 @@
 #include "arrow/ipc/reader.h"
 #include "arrow/status.h"
 #include "arrow/table.h"
+#include "arrow/util/string_builder.h"
 #include "arrow/util/uri.h"
 
 namespace arrow {
@@ -105,11 +106,11 @@ bool FlightDescriptor::Equals(const FlightDescriptor& other) const {
 
 std::string FlightDescriptor::ToString() const {
   std::stringstream ss;
-  ss << "FlightDescriptor<";
+  ss << "<FlightDescriptor ";
   switch (type) {
     case PATH: {
+      ss << "path='";
       bool first = true;
-      ss << "path = '";
       for (const auto& p : path) {
         if (!first) {
           ss << "/";
@@ -121,7 +122,7 @@ std::string FlightDescriptor::ToString() const {
       break;
     }
     case CMD:
-      ss << "cmd = '" << cmd << "'";
+      ss << "cmd='" << cmd << "'";
       break;
     default:
       break;
@@ -161,6 +162,10 @@ Status SchemaResult::GetSchema(ipc::DictionaryMemo* dictionary_memo,
   return GetSchema(dictionary_memo).Value(out);
 }
 
+std::string SchemaResult::ToString() const {
+  return "<SchemaResult raw_schema=(serialized)>";
+}
+
 bool SchemaResult::Equals(const SchemaResult& other) const {
   return raw_schema_ == other.raw_schema_;
 }
@@ -225,6 +230,12 @@ Status FlightDescriptor::Deserialize(const std::string& serialized,
   return Deserialize(serialized).Value(out);
 }
 
+std::string Ticket::ToString() const {
+  std::stringstream ss;
+  ss << "<Ticket ticket='" << ticket << "'>";
+  return ss.str();
+}
+
 bool Ticket::Equals(const Ticket& other) const { return ticket == other.ticket; }
 
 arrow::Result<std::string> Ticket::SerializeToString() const {
@@ -326,6 +337,36 @@ Status FlightInfo::Deserialize(const std::string& serialized,
   return Deserialize(serialized).Value(out);
 }
 
+std::string FlightInfo::ToString() const {
+  std::stringstream ss;
+  ss << "<FlightInfo schema=";
+  if (schema_) {
+    ss << schema_->ToString();
+  } else {
+    ss << "(serialized)";
+  }
+  ss << " descriptor=" << data_.descriptor.ToString();
+  ss << " endpoints=[";
+  bool first = true;
+  for (const auto& endpoint : data_.endpoints) {
+    if (!first) ss << ", ";
+    ss << endpoint.ToString();
+    first = false;
+  }
+  ss << "] total_records=" << data_.total_records;
+  ss << " total_bytes=" << data_.total_bytes;
+  ss << '>';
+  return ss.str();
+}
+
+bool FlightInfo::Equals(const FlightInfo& other) const {
+  return data_.schema == other.data_.schema &&
+         data_.descriptor == other.data_.descriptor &&
+         data_.endpoints == other.data_.endpoints &&
+         data_.total_records == other.data_.total_records &&
+         data_.total_bytes == other.data_.total_bytes;
+}
+
 Location::Location() { uri_ = std::make_shared<arrow::internal::Uri>(); }
 
 Status FlightListing::Next(std::unique_ptr<FlightInfo>* info) {
@@ -393,6 +434,20 @@ bool Location::Equals(const Location& other) const {
   return ToString() == other.ToString();
 }
 
+std::string FlightEndpoint::ToString() const {
+  std::stringstream ss;
+  ss << "<FlightEndpoint ticket=" << ticket.ToString();
+  ss << " locations=[";
+  bool first = true;
+  for (const auto& location : locations) {
+    if (!first) ss << ", ";
+    ss << location.ToString();
+    first = false;
+  }
+  ss << "]>";
+  return ss.str();
+}
+
 bool FlightEndpoint::Equals(const FlightEndpoint& other) const {
   return ticket == other.ticket && locations == other.locations;
 }
@@ -423,6 +478,11 @@ arrow::Result<FlightEndpoint> FlightEndpoint::Deserialize(std::string_view seria
   return out;
 }
 
+std::string ActionType::ToString() const {
+  return arrow::util::StringBuilder("<ActionType type='", type, "' description='",
+                                    description, "'>");
+}
+
 bool ActionType::Equals(const ActionType& other) const {
   return type == other.type && description == other.description;
 }
@@ -453,6 +513,10 @@ arrow::Result<ActionType> ActionType::Deserialize(std::string_view serialized) {
   return out;
 }
 
+std::string Criteria::ToString() const {
+  return arrow::util::StringBuilder("<Criteria expression='", expression, "'>");
+}
+
 bool Criteria::Equals(const Criteria& other) const {
   return expression == other.expression;
 }
@@ -483,6 +547,19 @@ arrow::Result<Criteria> Criteria::Deserialize(std::string_view serialized) {
   return out;
 }
 
+std::string Action::ToString() const {
+  std::stringstream ss;
+  ss << "<Action type='" << type;
+  ss << "' body=";
+  if (body) {
+    ss << "(" << body->size() << " bytes)";
+  } else {
+    ss << "(nullptr)";
+  }
+  ss << '>';
+  return ss.str();
+}
+
 bool Action::Equals(const Action& other) const {
   return (type == other.type) &&
          ((body == other.body) || (body && other.body && body->Equals(*other.body)));
@@ -514,6 +591,17 @@ arrow::Result<Action> Action::Deserialize(std::string_view serialized) {
   return out;
 }
 
+std::string Result::ToString() const {
+  std::stringstream ss;
+  ss << "<Result body=";
+  if (body) {
+    ss << "(" << body->size() << " bytes)>";
+  } else {
+    ss << "(nullptr)>";
+  }
+  return ss.str();
+}
+
 bool Result::Equals(const Result& other) const {
   return (body == other.body) || (body && other.body && body->Equals(*other.body));
 }
@@ -638,6 +726,11 @@ arrow::Result<std::unique_ptr<Result>> SimpleResultStream::Next() {
   return std::make_unique<Result>(std::move(results_[position_++]));
 }
 
+std::string BasicAuth::ToString() const {
+  return arrow::util::StringBuilder("<BasicAuth username='", username,
+                                    "' password=(redacted)>");
+}
+
 bool BasicAuth::Equals(const BasicAuth& other) const {
   return (username == other.username) && (password == other.password);
 }
