@@ -580,7 +580,14 @@ Result<std::shared_ptr<RecordBatch>> Table::CombineChunksToBatch(MemoryPool* poo
   ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Table> combined, CombineChunks(pool));
   std::vector<std::shared_ptr<Array>> arrays;
   for (const auto& column : combined->columns()) {
-    arrays.push_back(column->chunk(0));
+    if (column->num_chunks() == 0) {
+      DCHECK_EQ(num_rows(), 0) << "Empty chunk with more than 0 rows";
+      ARROW_ASSIGN_OR_RAISE(auto chunk,
+                            MakeArrayOfNull(column->type(), num_rows(), pool));
+      arrays.push_back(std::move(chunk));
+    } else {
+      arrays.push_back(column->chunk(0));
+    }
   }
   return RecordBatch::Make(schema_, num_rows_, std::move(arrays));
 }
