@@ -25,6 +25,7 @@
 #include "arrow/testing/util.h"
 #include "arrow/type.h"
 #include "arrow/util/byte_stream_split.h"
+#include "arrow/visit_data_inline.h"
 
 #include "parquet/encoding.h"
 #include "parquet/platform.h"
@@ -215,7 +216,7 @@ struct BM_SpacedEncodingTraits<BooleanType> {
   using CType = bool;
 };
 
-static void BM_PlainSpacedArgs(benchmark::internal::Benchmark* bench) {
+static void BM_SpacedArgs(benchmark::internal::Benchmark* bench) {
   constexpr auto kPlainSpacedSize = 32 * 1024;  // 32k
 
   bench->Args({/*size*/ kPlainSpacedSize, /*null_in_ten_thousand*/ 1});
@@ -226,7 +227,7 @@ static void BM_PlainSpacedArgs(benchmark::internal::Benchmark* bench) {
 }
 
 template <typename ParquetType>
-static void BM_PlainEncodingSpaced(benchmark::State& state) {
+static void BM_EncodingSpaced(benchmark::State& state, Encoding::type encoding) {
   using ArrowType = typename BM_SpacedEncodingTraits<ParquetType>::ArrowType;
   using ArrayType = typename BM_SpacedEncodingTraits<ParquetType>::ArrayType;
   using CType = typename BM_SpacedEncodingTraits<ParquetType>::CType;
@@ -245,7 +246,7 @@ static void BM_PlainEncodingSpaced(benchmark::State& state) {
   // input to PutSpaced.
   const auto src = reinterpret_cast<const CType*>(raw_values);
 
-  auto encoder = MakeTypedEncoder<ParquetType>(Encoding::PLAIN);
+  auto encoder = MakeTypedEncoder<ParquetType>(encoding);
   for (auto _ : state) {
     encoder->PutSpaced(src, num_values, valid_bits, 0);
     encoder->FlushValues();
@@ -254,23 +255,59 @@ static void BM_PlainEncodingSpaced(benchmark::State& state) {
   state.SetBytesProcessed(state.iterations() * num_values * sizeof(CType));
 }
 
+template <>
+void BM_EncodingSpaced<BooleanType>(benchmark::State& state, Encoding::type encoding) {
+  using CType = bool;
+
+  const int num_values = static_cast<int>(state.range(0));
+  const double null_percent = static_cast<double>(state.range(1)) / 10000.0;
+
+  auto rand = ::arrow::random::RandomArrayGenerator(1923);
+  const auto array = rand.Boolean(num_values, 0.5, null_percent);
+  const auto valid_bits = array->null_bitmap_data();
+  bool* output = new bool[state.range(0)];
+  int output_idx = 0;
+  PARQUET_THROW_NOT_OK(::arrow::VisitArraySpanInline<::arrow::BooleanType>(
+      *array->data(),
+      [&](bool value) {
+        output[output_idx] = value;
+        ++output_idx;
+        return ::arrow::Status::OK();
+      },
+      []() { return ::arrow::Status::OK(); }));
+
+  auto encoder = MakeTypedEncoder<BooleanType>(encoding);
+  for (auto _ : state) {
+    encoder->PutSpaced(output, num_values, valid_bits, 0);
+    encoder->FlushValues();
+  }
+  state.counters["null_percent"] = null_percent * 100;
+  state.SetBytesProcessed(state.iterations() * num_values * sizeof(CType));
+  delete[] output;
+}
+
+template <typename ParquetType>
+static void BM_PlainEncodingSpaced(benchmark::State& state) {
+  BM_EncodingSpaced<ParquetType>(state, Encoding::PLAIN);
+}
+
 static void BM_PlainEncodingSpacedBoolean(benchmark::State& state) {
   BM_PlainEncodingSpaced<BooleanType>(state);
 }
-BENCHMARK(BM_PlainEncodingSpacedBoolean)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainEncodingSpacedBoolean)->Apply(BM_SpacedArgs);
 
 static void BM_PlainEncodingSpacedFloat(benchmark::State& state) {
   BM_PlainEncodingSpaced<FloatType>(state);
 }
-BENCHMARK(BM_PlainEncodingSpacedFloat)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainEncodingSpacedFloat)->Apply(BM_SpacedArgs);
 
 static void BM_PlainEncodingSpacedDouble(benchmark::State& state) {
   BM_PlainEncodingSpaced<DoubleType>(state);
 }
-BENCHMARK(BM_PlainEncodingSpacedDouble)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainEncodingSpacedDouble)->Apply(BM_SpacedArgs);
 
 template <typename ParquetType>
-static void BM_PlainDecodingSpaced(benchmark::State& state) {
+static void BM_DecodingSpaced(benchmark::State& state, Encoding::type encoding) {
   using ArrowType = typename BM_SpacedEncodingTraits<ParquetType>::ArrowType;
   using ArrayType = typename BM_SpacedEncodingTraits<ParquetType>::ArrayType;
   using CType = typename BM_SpacedEncodingTraits<ParquetType>::CType;
@@ -279,22 +316,21 @@ static void BM_PlainDecodingSpaced(benchmark::State& state) {
   const auto null_percent = static_cast<double>(state.range(1)) / 10000.0;
 
   auto rand = ::arrow::random::RandomArrayGenerator(1923);
-  const auto array = rand.Numeric<ArrowType>(num_values, -100, 100, null_percent);
+  std::shared_ptr<::arrow::Array> array;
+  if constexpr (std::is_same_v<ParquetType, BooleanType>) {
+    array = rand.Boolean(num_values, /*true_probability*/ 0.5, null_percent);
+  } else {
+    array = rand.Numeric<ArrowType>(num_values, -100, 100, null_percent);
+  }
   const auto valid_bits = array->null_bitmap_data();
   const int null_count = static_cast<int>(array->null_count());
   const auto array_actual = ::arrow::internal::checked_pointer_cast<ArrayType>(array);
-  const auto raw_values = array_actual->raw_values();
-  // Guarantee the type cast between raw_values and input of PutSpaced.
-  static_assert(sizeof(CType) == sizeof(*raw_values), "Type mismatch");
-  // Cast only happens for BooleanType as it use UInt8 for the array data to match a bool*
-  // input to PutSpaced.
-  const auto src = reinterpret_cast<const CType*>(raw_values);
 
-  auto encoder = MakeTypedEncoder<ParquetType>(Encoding::PLAIN);
-  encoder->PutSpaced(src, num_values, valid_bits, 0);
+  auto encoder = MakeTypedEncoder<ParquetType>(encoding);
+  encoder->Put(*array);
   std::shared_ptr<Buffer> buf = encoder->FlushValues();
 
-  auto decoder = MakeTypedDecoder<ParquetType>(Encoding::PLAIN);
+  auto decoder = MakeTypedDecoder<ParquetType>(encoding);
   std::vector<uint8_t> decode_values(num_values * sizeof(CType));
   auto decode_buf = reinterpret_cast<CType*>(decode_values.data());
   for (auto _ : state) {
@@ -305,20 +341,25 @@ static void BM_PlainDecodingSpaced(benchmark::State& state) {
   state.SetBytesProcessed(state.iterations() * num_values * sizeof(CType));
 }
 
+template <typename ParquetType>
+static void BM_PlainDecodingSpaced(benchmark::State& state) {
+  BM_DecodingSpaced<ParquetType>(state, Encoding::PLAIN);
+}
+
 static void BM_PlainDecodingSpacedBoolean(benchmark::State& state) {
   BM_PlainDecodingSpaced<BooleanType>(state);
 }
-BENCHMARK(BM_PlainDecodingSpacedBoolean)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainDecodingSpacedBoolean)->Apply(BM_SpacedArgs);
 
 static void BM_PlainDecodingSpacedFloat(benchmark::State& state) {
   BM_PlainDecodingSpaced<FloatType>(state);
 }
-BENCHMARK(BM_PlainDecodingSpacedFloat)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainDecodingSpacedFloat)->Apply(BM_SpacedArgs);
 
 static void BM_PlainDecodingSpacedDouble(benchmark::State& state) {
   BM_PlainDecodingSpaced<DoubleType>(state);
 }
-BENCHMARK(BM_PlainDecodingSpacedDouble)->Apply(BM_PlainSpacedArgs);
+BENCHMARK(BM_PlainDecodingSpacedDouble)->Apply(BM_SpacedArgs);
 
 template <typename T, typename DecodeFunc>
 static void BM_ByteStreamSplitDecode(benchmark::State& state, DecodeFunc&& decode_func) {
@@ -528,8 +569,8 @@ static void BM_DeltaBitPackingDecode(benchmark::State& state, NumberGenerator ge
   encoder->Put(values.data(), static_cast<int>(values.size()));
   std::shared_ptr<Buffer> buf = encoder->FlushValues();
 
+  auto decoder = MakeTypedDecoder<DType>(Encoding::DELTA_BINARY_PACKED);
   for (auto _ : state) {
-    auto decoder = MakeTypedDecoder<DType>(Encoding::DELTA_BINARY_PACKED);
     decoder->SetData(static_cast<int>(values.size()), buf->data(),
                      static_cast<int>(buf->size()));
     decoder->Decode(values.data(), static_cast<int>(values.size()));
@@ -569,6 +610,178 @@ BENCHMARK(BM_DeltaBitPackingDecode_Int64_Narrow)->Range(MIN_RANGE, MAX_RANGE);
 BENCHMARK(BM_DeltaBitPackingDecode_Int32_Wide)->Range(MIN_RANGE, MAX_RANGE);
 BENCHMARK(BM_DeltaBitPackingDecode_Int64_Wide)->Range(MIN_RANGE, MAX_RANGE);
 
+static void ByteArrayCustomArguments(benchmark::internal::Benchmark* b) {
+  b->ArgsProduct({{8, 64, 1024}, {512, 2048}})
+      ->ArgNames({"max-string-length", "batch-size"});
+}
+
+void EncodingByteArrayBenchmark(benchmark::State& state, Encoding::type encoding) {
+  ::arrow::random::RandomArrayGenerator rag(0);
+  // Using arrow generator to generate random data.
+  int32_t max_length = static_cast<int32_t>(state.range(0));
+  int32_t array_size = static_cast<int32_t>(state.range(1));
+  auto array =
+      rag.String(/* size */ array_size, /* min_length */ 0, /* max_length */ max_length,
+                 /* null_probability */ 0);
+  const auto array_actual =
+      ::arrow::internal::checked_pointer_cast<::arrow::StringArray>(array);
+  auto encoder = MakeTypedEncoder<ByteArrayType>(encoding);
+  std::vector<ByteArray> values;
+  for (int i = 0; i < array_actual->length(); ++i) {
+    values.emplace_back(array_actual->GetView(i));
+  }
+
+  for (auto _ : state) {
+    encoder->Put(values.data(), static_cast<int>(values.size()));
+    encoder->FlushValues();
+  }
+  state.SetItemsProcessed(state.iterations() * array_actual->length());
+  state.SetBytesProcessed(state.iterations() * (array_actual->value_data()->size() +
+                                                array_actual->value_offsets()->size()));
+}
+
+static void BM_DeltaLengthEncodingByteArray(benchmark::State& state) {
+  EncodingByteArrayBenchmark(state, Encoding::DELTA_LENGTH_BYTE_ARRAY);
+}
+
+static void BM_PlainEncodingByteArray(benchmark::State& state) {
+  EncodingByteArrayBenchmark(state, Encoding::PLAIN);
+}
+
+void DecodingByteArrayBenchmark(benchmark::State& state, Encoding::type encoding) {
+  ::arrow::random::RandomArrayGenerator rag(0);
+  int32_t max_length = static_cast<int32_t>(state.range(0));
+  int32_t array_size = static_cast<int32_t>(state.range(1));
+  // Using arrow to write, because we just benchmark decoding here.
+  auto array =
+      rag.String(/* size */ array_size, /* min_length */ 0, /* max_length */ max_length,
+                 /* null_probability */ 0);
+  const auto array_actual =
+      ::arrow::internal::checked_pointer_cast<::arrow::StringArray>(array);
+  auto encoder = MakeTypedEncoder<ByteArrayType>(encoding);
+  encoder->Put(*array);
+  std::shared_ptr<Buffer> buf = encoder->FlushValues();
+
+  std::vector<ByteArray> values;
+  values.resize(array->length());
+  for (auto _ : state) {
+    auto decoder = MakeTypedDecoder<ByteArrayType>(encoding);
+    decoder->SetData(static_cast<int>(array->length()), buf->data(),
+                     static_cast<int>(buf->size()));
+    decoder->Decode(values.data(), static_cast<int>(values.size()));
+    ::benchmark::DoNotOptimize(values);
+  }
+  state.SetItemsProcessed(state.iterations() * array->length());
+  state.SetBytesProcessed(state.iterations() * (array_actual->value_data()->size() +
+                                                array_actual->value_offsets()->size()));
+}
+
+static void BM_PlainDecodingByteArray(benchmark::State& state) {
+  DecodingByteArrayBenchmark(state, Encoding::PLAIN);
+}
+
+static void BM_DeltaLengthDecodingByteArray(benchmark::State& state) {
+  DecodingByteArrayBenchmark(state, Encoding::DELTA_LENGTH_BYTE_ARRAY);
+}
+
+BENCHMARK(BM_PlainEncodingByteArray)->Apply(ByteArrayCustomArguments);
+BENCHMARK(BM_DeltaLengthEncodingByteArray)->Apply(ByteArrayCustomArguments);
+BENCHMARK(BM_PlainDecodingByteArray)->Apply(ByteArrayCustomArguments);
+BENCHMARK(BM_DeltaLengthDecodingByteArray)->Apply(ByteArrayCustomArguments);
+
+static void BM_DecodingByteArraySpaced(benchmark::State& state, Encoding::type encoding) {
+  const double null_percent = 0.02;
+
+  auto rand = ::arrow::random::RandomArrayGenerator(0);
+  int32_t max_length = static_cast<int32_t>(state.range(0));
+  int32_t num_values = static_cast<int32_t>(state.range(1));
+  const auto array = rand.String(num_values, /* min_length */ 0,
+                                 /* max_length */ max_length, null_percent);
+  const auto valid_bits = array->null_bitmap_data();
+  const int null_count = static_cast<int>(array->null_count());
+  const auto array_actual =
+      ::arrow::internal::checked_pointer_cast<::arrow::StringArray>(array);
+
+  std::vector<ByteArray> byte_arrays;
+  byte_arrays.reserve(array_actual->length());
+  for (int i = 0; i < array_actual->length(); ++i) {
+    byte_arrays.emplace_back(array_actual->GetView(i));
+  }
+
+  auto encoder = MakeTypedEncoder<ByteArrayType>(encoding);
+  encoder->PutSpaced(byte_arrays.data(), num_values, valid_bits, 0);
+  std::shared_ptr<Buffer> buf = encoder->FlushValues();
+
+  auto decoder = MakeTypedDecoder<ByteArrayType>(encoding);
+  std::vector<uint8_t> decode_values(num_values * sizeof(ByteArray));
+  auto decode_buf = reinterpret_cast<ByteArray*>(decode_values.data());
+  for (auto _ : state) {
+    decoder->SetData(num_values - null_count, buf->data(), static_cast<int>(buf->size()));
+    decoder->DecodeSpaced(decode_buf, num_values, null_count, valid_bits, 0);
+    ::benchmark::DoNotOptimize(decode_buf);
+  }
+  state.counters["null_percent"] = null_percent * 100;
+  state.SetItemsProcessed(state.iterations() * array_actual->length());
+  state.SetBytesProcessed(state.iterations() * (array_actual->value_data()->size() +
+                                                array_actual->value_offsets()->size()));
+}
+
+static void BM_PlainDecodingSpacedByteArray(benchmark::State& state) {
+  BM_DecodingByteArraySpaced(state, Encoding::PLAIN);
+}
+
+static void BM_DeltaLengthDecodingSpacedByteArray(benchmark::State& state) {
+  BM_DecodingByteArraySpaced(state, Encoding::DELTA_LENGTH_BYTE_ARRAY);
+}
+
+BENCHMARK(BM_PlainDecodingSpacedByteArray)->Apply(ByteArrayCustomArguments);
+BENCHMARK(BM_DeltaLengthDecodingSpacedByteArray)->Apply(ByteArrayCustomArguments);
+
+static void BM_RleEncodingBoolean(benchmark::State& state) {
+  std::vector<bool> values(state.range(0), true);
+  auto encoder = MakeEncoder(Type::BOOLEAN, Encoding::RLE);
+  auto typed_encoder = dynamic_cast<BooleanEncoder*>(encoder.get());
+
+  for (auto _ : state) {
+    typed_encoder->Put(values, static_cast<int>(values.size()));
+    typed_encoder->FlushValues();
+  }
+  state.SetBytesProcessed(state.iterations() * state.range(0) * sizeof(bool));
+}
+
+BENCHMARK(BM_RleEncodingBoolean)->Range(MIN_RANGE, MAX_RANGE);
+
+static void BM_RleDecodingBoolean(benchmark::State& state) {
+  std::vector<bool> values(state.range(0), true);
+  bool* output = new bool[state.range(0)];
+  auto encoder = MakeEncoder(Type::BOOLEAN, Encoding::RLE);
+  auto typed_encoder = dynamic_cast<BooleanEncoder*>(encoder.get());
+  typed_encoder->Put(values, static_cast<int>(values.size()));
+  std::shared_ptr<Buffer> buf = encoder->FlushValues();
+
+  auto decoder = MakeTypedDecoder<BooleanType>(Encoding::RLE);
+  for (auto _ : state) {
+    decoder->SetData(static_cast<int>(values.size()), buf->data(),
+                     static_cast<int>(buf->size()));
+    decoder->Decode(output, static_cast<int>(values.size()));
+  }
+
+  state.SetBytesProcessed(state.iterations() * state.range(0) * sizeof(bool));
+  delete[] output;
+}
+
+BENCHMARK(BM_RleDecodingBoolean)->Range(MIN_RANGE, MAX_RANGE);
+
+static void BM_RleEncodingSpacedBoolean(benchmark::State& state) {
+  BM_EncodingSpaced<BooleanType>(state, Encoding::RLE);
+}
+BENCHMARK(BM_RleEncodingSpacedBoolean)->Apply(BM_SpacedArgs);
+
+static void BM_RleDecodingSpacedBoolean(benchmark::State& state) {
+  BM_DecodingSpaced<BooleanType>(state, Encoding::RLE);
+}
+BENCHMARK(BM_RleDecodingSpacedBoolean)->Apply(BM_SpacedArgs);
+
 template <typename Type>
 static void DecodeDict(std::vector<typename Type::c_type>& values,
                        benchmark::State& state) {
@@ -634,6 +847,29 @@ static void BM_DictDecodingInt64_literals(benchmark::State& state) {
 
 BENCHMARK(BM_DictDecodingInt64_literals)->Range(MIN_RANGE, MAX_RANGE);
 
+static void BM_DictDecodingByteArray(benchmark::State& state) {
+  ::arrow::random::RandomArrayGenerator rag(0);
+  // Using arrow generator to generate random data.
+  int32_t max_length = static_cast<int32_t>(state.range(0));
+  int32_t array_size = static_cast<int32_t>(state.range(1));
+  auto array =
+      rag.String(/* size */ array_size, /* min_length */ 0, /* max_length */ max_length,
+                 /* null_probability */ 0);
+  const auto array_actual =
+      ::arrow::internal::checked_pointer_cast<::arrow::StringArray>(array);
+  auto encoder = MakeDictDecoder<ByteArrayType>();
+  std::vector<ByteArray> values;
+  for (int i = 0; i < array_actual->length(); ++i) {
+    values.emplace_back(array_actual->GetView(i));
+  }
+  DecodeDict<ByteArrayType>(values, state);
+  state.SetItemsProcessed(state.iterations() * array_actual->length());
+  state.SetBytesProcessed(state.iterations() * (array_actual->value_data()->size() +
+                                                array_actual->value_offsets()->size()));
+}
+
+BENCHMARK(BM_DictDecodingByteArray)->Apply(ByteArrayCustomArguments);
+
 // ----------------------------------------------------------------------
 // Shared benchmarks for decoding using arrow builders
 
